==> json-joy-2.9.1.tgz: package/esm/json-rx/JsonRxServer.js <==
import { asString } from 'json-schema-serializer';
import { from, isObservable, of } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { assertId, assertName, isArray, microtask } from './util';
import { TimedQueue } from './TimedQueue';
const formatError = (error) => {
    let json = '{"message":' + asString(error.message);
    if (typeof error.status === 'number')
        json += ',"status":' + error.status;
    if (typeof error.code === 'string')
        json += ',"code":' + asString(error.code);
    if (typeof error.errno === 'number')
        json += ',"errno":' + error.errno;
    if (typeof error.errorId === 'string')
        json += ',"errorId":' + asString(error.errorId);
    return (json + '}');
};
export class JsonRxServer {
    constructor({ send, call, notify, maxActiveSubscriptions = 30, bufferSize = 10, bufferTime = 1, }) {
        this.active = new Map();
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
        if (bufferTime) {
            const buffer = new TimedQueue();
            buffer.itemLimit = bufferSize;
            buffer.timeLimit = bufferTime;
            buffer.onFlush = (messages) => {
                send(messages.length === 1 ? messages[0] : ('[' + messages.join(',') + ']'));
            };
            this.send = (message) => buffer.push(message);
        }
        else {
            this.send = send;
        }
    }
    sendError(id, error) {
        const message = ('[-1,' + id + ',' + JSON.stringify(error) + ']');
        this.send(message);
    }
    onSubscribe(message, ctx) {
        const [id, name, payload] = message;
        assertId(id);
        assertName(name);
        try {
            const active = this.active.get(id);
            if (active) {
                this.active.delete(id);
                active.unsubscribe();
                this.send(('[-1,' + id + ',{"message":"ID already active."}]'));
                return;
            }
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            const callResult = this.call(name, payload, ctx);
            const observable = isObservable(callResult)
                ? callResult
                : from(callResult).pipe(mergeMap((value) => (isObservable(value) ? value : of(value))));
            const ref = { buffer: [] };
            let done = false;
            const subscription = observable.subscribe((data) => {
                if (data === undefined)
                    return;
                ref.buffer.push(data);
                microtask(() => {
                    if (!ref.buffer.length)
                        return;
                    try {
                        for (const payload of ref.buffer)
                            this.send(('[-2,' + id + ',' + payload + ']'));
                    }
                    finally {
                        ref.buffer = [];
                    }
                });
            }, (error) => {
                done = true;
                this.active.delete(id);
                let message;
                if (error instanceof Error) {
                    message = ('[-1,' + id + ',' + formatError(error) + ']');
                }
                else {
                    message = ('[-1,' + id + ',' + JSON.stringify(error) + ']');
                }
                this.send(message);
            }, () => {
                done = true;
                this.active.delete(id);
                try {
                    if (!ref.buffer.length)
                        return this.send(`[0,${id}]`);
                    const last = ref.buffer.length - 1;
                    for (let i = 0; i <= last; i++) {
                        const isLast = i === last;
                        let message;
                        if (isLast) {
                            message = ('[0,' + id + ',' + ref.buffer[i] + ']');
                        }
                        else {
                            message = ('[-2,' + id + ',' + ref.buffer[i] + ']');
                        }
                        this.send(message);
                    }
                }
                finally {
                    ref.buffer = [];
                }
            });
            if (!done)
                this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    }
    onUnsubscribe(message) {
        const [, id] = message;
        assertId(id);
        const subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    }
    onNotification(message, ctx) {
        const [name, payload] = message;
        assertName(name);
        this.notify(name, payload, ctx);
    }
    onMessage(message, ctx) {
        if (!isArray(message))
            throw new Error('Invalid message');
        const [one] = message;
        if (isArray(one)) {
            for (let i = 0; i < message.length; i++)
                this.onMessage(message[i], ctx);
            return;
        }
        if (typeof one === 'string')
            return this.onNotification(message, ctx);
        if (one > 0)
            return this.onSubscribe(message, ctx);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    }
    stop() {
        this.send = (message) => { };
        for (const sub of this.active.values()) {
            sub.unsubscribe();
        }
    }
}

==> json-joy-2.17.0.tgz: package/esm/json-rx/JsonRxServer.js <==
import { from, isObservable, of } from 'rxjs';
import { mergeMap } from 'rxjs/operators';
import { assertId, assertName, isArray, microtask } from './util';
import { TimedQueue } from './TimedQueue';
const formatError = (error) => {
    let obj = { message: error.message };
    if (typeof error.status === 'number')
        obj.status = error.status;
    if (typeof error.code === 'string')
        obj.code = error.code;
    if (typeof error.errno === 'number')
        obj.errno = error.errno;
    if (typeof error.errorId === 'string')
        obj.errorId = error.errorId;
    return obj;
};
export class JsonRxServer {
    constructor({ send, call, notify, maxActiveSubscriptions = 30, bufferSize = 10, bufferTime = 1, }) {
        this.active = new Map();
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
        if (bufferTime) {
            const buffer = new TimedQueue();
            buffer.itemLimit = bufferSize;
            buffer.timeLimit = bufferTime;
            buffer.onFlush = (messages) => {
                send(messages.length === 1 ? messages[0] : messages);
            };
            this.send = (message) => buffer.push(message);
        }
        else {
            this.send = send;
        }
    }
    sendError(id, error) {
        const message = ([-1, id, error]);
        this.send(message);
    }
    onSubscribe(message, ctx) {
        const [id, name, payload] = message;
        assertId(id);
        assertName(name);
        try {
            const active = this.active.get(id);
            if (active) {
                this.active.delete(id);
                active.unsubscribe();
                this.send(([-1, id, { message: "ID already active." }]));
                return;
            }
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            const callResult = this.call(name, payload, ctx);
            const observable = isObservable(callResult)
                ? callResult
                : from(callResult).pipe(mergeMap((value) => (isObservable(value) ? value : of(value))));
            const ref = { buffer: [] };
            let done = false;
            const subscription = observable.subscribe((data) => {
                if (data === undefined)
                    return;
                ref.buffer.push(data);
                microtask(() => {
                    if (!ref.buffer.length)
                        return;
                    try {
                        for (const payload of ref.buffer)
                            this.send(([-2, id, payload]));
                    }
                    finally {
                        ref.buffer = [];
                    }
                });
            }, (error) => {
                done = true;
                this.active.delete(id);
                let message;
                if (error instanceof Error) {
                    message = ([-1, id, formatError(error)]);
                }
                else {
                    message = ([-1, id, error]);
                }
                this.send(message);
            }, () => {
                done = true;
                this.active.delete(id);
                try {
                    if (!ref.buffer.length)
                        return this.send([0, id]);
                    const last = ref.buffer.length - 1;
                    for (let i = 0; i <= last; i++) {
                        const isLast = i === last;
                        let message;
                        if (isLast) {
                            message = ([0, id, ref.buffer[i]]);
                        }
                        else {
                            message = ([-2, id, ref.buffer[i]]);
                        }
                        this.send(message);
                    }
                }
                finally {
                    ref.buffer = [];
                }
            });
            if (!done)
                this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    }
    onUnsubscribe(message) {
        const [, id] = message;
        assertId(id);
        const subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    }
    onNotification(message, ctx) {
        const [name, payload] = message;
        assertName(name);
        this.notify(name, payload, ctx);
    }
    onMessage(message, ctx) {
        if (!isArray(message))
            throw new Error('Invalid message');
        const [one] = message;
        if (isArray(one)) {
            for (let i = 0; i < message.length; i++)
                this.onMessage(message[i], ctx);
            return;
        }
        if (typeof one === 'string')
            return this.onNotification(message, ctx);
        if (one > 0)
            return this.onSubscribe(message, ctx);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    }
    stop() {
        this.send = (message) => { };
        for (const sub of this.active.values()) {
            sub.unsubscribe();
        }
    }
}
